SystemOrganization addCategory: #'Designer-Core'!

Morph subclass: #UiDesigner
	instanceVariableNames: 'widgetList container propertyTable categoryEdit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Designer-Core'!

!UiDesigner class methodsFor: 'open' stamp: 'mt 6/30/2010 09:13:56.969'!
open

	self new open.! !

!UiDesigner methodsFor: 'updating' stamp: 'mt 7/1/2010 17:41:45.299'!
changeWidgetNameFilter: aString

	widgetList model nameFilter: aString.! !

!UiDesigner methodsFor: 'initialization' stamp: 'mt 10/19/2010 14:36:57.283'!
createMainContainer

	| main widgetListContainer filterRow filterEdit |	
	main := Morph new
		color: (Color gray: 0.8);
		changeTableLayout;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #spaceFill.
	
	widgetListContainer := Morph new
		color: Color transparent;
		changeTableLayout;
		listDirection: #topToBottom;
		hResizing: #shrinkWrap;
		vResizing: #spaceFill.
		
	filterRow := Morph new
		color: Color transparent;
		changeTableLayout;
		listDirection: #leftToRight;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		layoutInset: 1;
		cellInset: 4.
	
	filterEdit := UiLineEdit new hResizing: #spaceFill.

	self connect: filterEdit signal: #textEdited: toSelector: #changeWidgetNameFilter:.

	filterRow
		addMorphBack: (UiLabel new form: UiFugueIcons magnifierIcon; setBalloonText: 'Search');
		addMorphBack: filterEdit.
	
	widgetListContainer
		addMorphBack: widgetList;
		addMorphBack: filterRow.
	
	main
		addMorphBack: widgetListContainer;
		addMorphBack: container;
		addMorphBack: (propertyTable withScrollBar vResizing: #spaceFill).
	
	^ main! !

!UiDesigner methodsFor: 'initialization' stamp: 'mt 7/4/2010 09:02:17.673'!
createToolbar

	| toolbar newBtn saveBtn loadBtn spacer |
	toolbar := Morph new
		color: (Color gray: 0.8);
		changeTableLayout;
		listDirection: #leftToRight;
		cellInset: 4;
		layoutInset: 4;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap.

	newBtn := UiToolButton new text: 'New'; buttonStyle: #textUnderIcon; autoRaise: true; cellInset: 1; icon: UiSilkIcons pageWhiteIcon.
	saveBtn := UiToolButton new text: 'Save'; buttonStyle: #textUnderIcon; autoRaise: true; cellInset: 1; icon: UiSilkIcons diskIcon.
	loadBtn := UiToolButton new text: 'Open'; buttonStyle: #textUnderIcon; autoRaise: true; cellInset: 1; icon: UiSilkIcons folderIcon.

	newBtn setBalloonText: 'Create a new UI.'.
	saveBtn setBalloonText: 'Save this UI. Generates code for a new UI class'.
	loadBtn setBalloonText: 'Load an existing UI class.'.

	self connect: newBtn signal: #clicked: toSelector: #reset.
	self connect: saveBtn signal: #clicked: toSelector: #save.
	self connect: loadBtn signal: #clicked: toSelector: #load.

	toolbar
		addMorphBack: newBtn;
		addMorphBack: loadBtn;
		addMorphBack: saveBtn.

	categoryEdit := UiLineEdit new theText: 'Widgets-Custom'; hResizing: #rigid; width: 150.
	spacer := UiContainer new
		hResizing: #spaceFill;
		height: 5.
		
	toolbar
		addMorphBack: spacer;
		addMorphBack: (UiLabel new form: UiSilkIcons packageIcon);
		addMorphBack: categoryEdit.

	^ toolbar! !

!UiDesigner methodsFor: 'initialization' stamp: 'mt 10/19/2010 16:34:13.981'!
expandDefaultWidgets

	1 to: (widgetList model rowCountFor: nil) - 1 do: [:i |
		widgetList toggleTopLevelRow: i].! !

!UiDesigner methodsFor: 'background' stamp: 'mt 6/30/2010 11:30:47.585'!
gridForm

	| form canvas |
	form := Form extent: 20@20 depth: 32.
	form floodFill: (Color gray: 0.8) at: 0@0.
	
	canvas := FormCanvas on: form.
	
	canvas point: 00@0 color: (Color gray: 0.3).
	
	^ form	! !

!UiDesigner methodsFor: 'initialization' stamp: 'mt 10/23/2010 10:00:20.145'!
initialize

	super initialize.
	self
		color: (Color gray: 0.4);
		changeTableLayout;
		listDirection: #topToBottom;
		extent: 800@600.
		
	widgetList := UiTreeView new
		removeHorizontalHeader;
		model: UiWidgetListModel new;
		selectionMode: UiViewNullSelection;
		vResizing: #spaceFill;
		width: 180.
	1 to: (widgetList model rowCountFor: nil) - 1 do: [:i |
		widgetList toggleTopLevelRow: i].
	
	"BAD: Viewport should not be accessed from the outside."
	widgetList viewport dragEnabled: true.
		
	propertyTable := UiPropertyTable new.
	
	"New empty container."
	container := UiContainer new.
	self initializeContainer.
	
	self
		addMorphBack: self createToolbar;
		addMorphBack: self createMainContainer.! !

!UiDesigner methodsFor: 'initialization' stamp: 'mt 6/30/2010 12:13:42.766'!
initializeContainer

	container
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		editMode: true;
		dragEnabled: false;
		fillStyle: (BitmapFillStyle fromForm: self gridForm).
		
	self connect: container signal: #selectionChanged: to: propertyTable selector: #updateProperties:.! !

!UiDesigner methodsFor: 'save/load' stamp: 'mt 10/23/2010 09:03:19.297'!
load

	| result ui |
	propertyTable clearCache.
	
	result := ListChooser
		chooseItemFrom: UserInterface subclasses
		title: 'Select UI class'.
	
	result ifNil: [^ self].
	
	ui := result new.
	ui initializePanel.
	
	container:= ui panel.
	self initializeContainer.
	
	self submorphs second submorphs second delete.
	self submorphs second addMorph: container after: self submorphs second submorphs first.
	
	categoryEdit theText: result category.
	
	propertyTable updateProperties: container.! !

!UiDesigner methodsFor: 'initialization' stamp: 'mt 6/30/2010 09:13:50.348'!
open

	self openInWindowLabeled: 'Morphic Designer'.! !

!UiDesigner methodsFor: 'save/load' stamp: 'mt 10/23/2010 09:03:25.09'!
reset

	"New empty container."
	propertyTable clearCache.
	
	container := UiContainer new.
	self initializeContainer.
	
	self submorphs second submorphs second delete.
	self submorphs second addMorph: container after: self submorphs second submorphs first.
	
	propertyTable updateProperties: container.! !

!UiDesigner methodsFor: 'save/load' stamp: 'mt 10/19/2010 16:35:14.364'!
save
	
	| generator |
	generator := UiCodeGenerator new
		category: categoryEdit theText;
		source: container.
	
	(Smalltalk includesKey: generator uiClassName asSymbol)
		ifTrue: [(UserDialogBoxMorph
			confirm:  'The UI class already exists:', String cr, '[', generator category, '] ', generator uiClassName
			title: 'Replace UI class?'
			trueChoice: 'Replace UI class.'
			falseChoice: 'Cancel.'
			at: self center) ifFalse: [^ self]].
	
	generator generateUiClassFor: container.
	self expandDefaultWidgets.
			
	UserDialogBoxMorph
		inform: 'UI class was created successfully to:', String cr, '[', generator category, '] ', generator uiClassName
		title: 'Done.'
		at: self center.	
! !

Morph subclass: #UiPropertyTable
	instanceVariableNames: 'nextRowColor lastMorph cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Designer-Core'!

!UiPropertyTable class methodsFor: 'defaults' stamp: 'mt 6/30/2010 09:02:58.262'!
defaultWidth

	^ 200! !

!UiPropertyTable methodsFor: 'adding - combined' stamp: 'mt 10/19/2010 16:45:51.177'!
addAllProperties: aMorph

	| editor |
	self addSeparator: 'General'.
	
	self addPropertyLabel: 'Type' editor: (UiLabel new text: aMorph class asString).
	
	((aMorph isKindOf: UiContainer) and: [(aMorph owner isKindOf: UiContainer) not])
		ifTrue: [
			editor := self createEditorFor: String value: aMorph uiClassName.
			self addPropertyLabel: 'UI Name' editor: editor.
			editor connect: editor signal: #valueChanged: to: aMorph selector: #uiClassName:.]
		ifFalse: [
			editor := self createEditorFor: String value: aMorph objectName.
			editor helpText: 'Type accessor here...'.
			self addPropertyLabel: #Name editor: editor.
			editor connect: editor signal: #valueChanged: to: aMorph selector: #objectName:].
		
	editor := self createEditorFor: String value: aMorph objectDescription.
	editor helpText: 'Type comment here...'.
	self addPropertyLabel: #Description editor: editor.
	editor connect: editor signal: #valueChanged: to: aMorph selector: #objectDescription:.
	
	self addSeparator: 'Geometry'.
	
"	self addIntegerProperty: #left morph: aMorph."
	self addPointProperty: #topLeft morph: aMorph.
	self addIntegerProperty: #width morph: aMorph.
	self addIntegerProperty: #height morph: aMorph.
	
	self addSeparator: 'General Layout'.
	
	self addListProperty: #hResizing values: #(rigid spaceFill shrinkWrap) morph: aMorph.
	self addListProperty: #vResizing values: #(rigid spaceFill shrinkWrap) morph: aMorph.
	
	editor := self createEditorFor: Boolean value: (aMorph layoutPolicy notNil and: [aMorph layoutPolicy isTableLayout]).
	self addPropertyLabel: 'Table Layout' editor: editor.
	self connect: editor signal: #toggled: toSelector: #changeTableLayout:enable: pattern: {#=. aMorph. 1}.
	
	aMorph hasTableLayout ifTrue: [self addTableLayoutProperties: aMorph].
	
	self addCustomProperties: aMorph.! !

!UiPropertyTable methodsFor: 'adding - combined' stamp: 'mt 10/21/2010 22:05:45.721'!
addCustomProperties: aMorph

	| editor |
	aMorph class widgetProperties do: [:category |
		self addSeparator: category key first.
		category value do: [:property | |target|
			target := aMorph perform: category key second.
			editor := self createEditorFor: property value value: (target perform: property key).
			self addPropertyLabel: property key editor: editor.

			"Post editor setup."
			({Integer. Point. String. Float} includes: property value)
				ifTrue: [self connect: editor signal: #valueChanged: to: target selector: (property key, #:) asSymbol].
			(property value = Boolean)
				ifTrue: [self connect: editor signal: #toggled: to: target selector: (property key, #:) asSymbol].
			(property value = Form)
				ifTrue: [self connect: editor signal: #clicked: toSelector: #pickIcon:selector:editor:
					pattern: {
						#=. aMorph. "will get the new icon"
						#=. (property key, #:) asSymbol. "the accessor to set the new icon"
						#=. editor "the editor that shows a preview of the icon"}].
			(property value isKindOf: Collection)
				ifTrue: [
					editor converter: [:item | item asSymbol]. "Treat elements as symbols."
					self connect: editor signal: #currentIndexChanged:text: to: target selector: (property key, #:) asSymbol pattern: #(2)]]].! !

!UiPropertyTable methodsFor: 'adding' stamp: 'mt 7/16/2010 11:05:23.169'!
addIntegerProperty: aProperty morph: aMorph

	| editor |
	editor := self createEditorFor: Integer value: (aMorph perform: aProperty).
	self addPropertyLabel: aProperty editor: editor.
	self connect: editor signal: #valueChanged: to: aMorph selector: (aProperty, #:) asSymbol.! !

!UiPropertyTable methodsFor: 'adding' stamp: 'mt 7/16/2010 11:11:21.729'!
addListProperty: aProperty values: someValues morph: aMorph
	"Treat list elements as symbols."
	
	| editor |
	editor := self createEditorFor: someValues value: (aMorph perform: aProperty).
	self addPropertyLabel: aProperty editor: editor.
	editor converter: [:item | item asSymbol].
	self connect: editor signal: #currentIndexChanged:text: to: aMorph selector: (aProperty, #:) asSymbol pattern: #(2).! !

!UiPropertyTable methodsFor: 'adding' stamp: 'mt 7/16/2010 11:05:53.434'!
addPointProperty: aProperty morph: aMorph

	| editor |
	editor := self createEditorFor: Point value: (aMorph perform: aProperty).
	self addPropertyLabel: aProperty editor: editor.
	self connect: editor signal: #valueChanged: to: aMorph selector: (aProperty, #:) asSymbol.! !

!UiPropertyTable methodsFor: 'private' stamp: 'mt 7/16/2010 11:00:34.529'!
addPropertyLabel: aString editor: aMorph

	| row |
	row := Morph new
		color: self nextRowColor;
		changeTableLayout;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		listDirection: #leftToRight;
		cellInset: 6;
		minHeight: 22;
		layoutInset: 1.

	row
		addMorphBack: (UiLabel new text: aString);
		addMorphBack: aMorph.
	
	self addMorphBack: row.! !

!UiPropertyTable methodsFor: 'private' stamp: 'mt 7/1/2010 10:46:55.99'!
addSeparator: aString

	| row |
	row := Morph new
		color: (Color gray: 0.4);
		changeTableLayout;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap;
		listDirection: #leftToRight;
		listCentering: #center;
		cellInset: 6;
		minHeight: 10;
		layoutInset: 1.

	row addMorphBack: (UiLabel new text: ('<font color="#FFFFFF" size="7"><b>', aString, '</b></font>') asHtmlText).
	self addMorphBack: row.
	
	self resized.! !

!UiPropertyTable methodsFor: 'adding' stamp: 'mt 7/16/2010 11:18:51.184'!
addStringProperty: aProperty morph: aMorph

	| editor |
	editor := self createEditorFor: String value: (aMorph perform: aProperty).
	self addPropertyLabel: aProperty editor: editor.
	self connect: editor signal: #valueChanged: to: aMorph selector: (aProperty, #:) asSymbol.! !

!UiPropertyTable methodsFor: 'adding - combined' stamp: 'mt 7/16/2010 11:12:03.569'!
addTableLayoutProperties: aMorph

	self addSeparator: 'Table Layout'.
	
	self addListProperty: #listDirection values: #(leftToRight rightToLeft topToBottom bottomToTop) morph: aMorph.

	self addIntegerProperty: #cellInset morph: aMorph.
	self addIntegerProperty: #layoutInset morph: aMorph.

	self addListProperty: #cellPositioning values: #(topLeft topRight bottomLeft bottomRight topCenter leftCenter rightCenter bottomCenter center) morph: aMorph.
	self addListProperty: #cellSpacing values: #(globalRect globalSquare localRect localSquare none) morph: aMorph.
	self addListProperty: #listCentering values: #(topLeft bottomRight center justified) morph: aMorph.
	self addListProperty: #listSpacing values: #(equal none) morph: aMorph.
	
	self addPointProperty: #minCellSize morph: aMorph.
	self addPointProperty: #maxCellSize morph: aMorph.
	
	self addListProperty: #wrapCentering values: #(topLeft bottomRight center justified) morph: aMorph.
	self addListProperty: #wrapDirection values: #(leftToRight rightToLeft topToBottom bottomToTop none) morph: aMorph.! !

!UiPropertyTable methodsFor: 'updating' stamp: 'mt 10/23/2010 09:05:56.725'!
changeTableLayout: aMorph enable: aBoolean
	"More complex callback to change the layout of a morph."
	
	aBoolean
		ifTrue: [aMorph changeTableLayout.
			lastMorph := nil.
			cache removeKey: aMorph ifAbsent: [].
			self updateProperties: aMorph]
		ifFalse: [aMorph layoutPolicy: nil; layoutChanged.
			lastMorph := nil.
			cache removeKey: aMorph ifAbsent: [].
			self updateProperties: aMorph]! !

!UiPropertyTable methodsFor: 'initialization' stamp: 'mt 10/23/2010 08:59:12.578'!
clearCache

	cache := IdentityDictionary new.! !

!UiPropertyTable methodsFor: 'private' stamp: 'mt 10/21/2010 22:15:19.2'!
createEditorFor: listOrClass value: aValue
	"Create the corresponding editor morph and set a start value."
	
	| editor |

	listOrClass == Boolean
		ifTrue: [editor := UiCheckBox new icon: nil; text: nil; checked: aValue].
	listOrClass == String
		ifTrue: [editor := UiLineEdit new theText: aValue.
			editor hResizing: #spaceFill].
	listOrClass == Integer
		ifTrue: [editor := UiLineEdit new theText: aValue; validator: [:raw | raw asInteger ifNil: [0]].
			editor hResizing: #spaceFill].
	listOrClass == Point
		ifTrue: [editor := UiLineEdit new theText: aValue;
			validator: [:raw | |tokens result|
				tokens := raw findTokens: #@.
				tokens ifEmpty: [result := 0@0].
				tokens size = 1 ifTrue: [result := tokens first asInteger @ 0].
				result ifNil: [tokens first asInteger @ tokens second asInteger]];
			hResizing: #spaceFill].
	listOrClass == Float
		ifTrue: [editor := UiLineEdit new theText: aValue; validator: [:raw | [(Float readFrom: raw readStream)] on: Error do: [:e | 0.0]].
			editor hResizing: #spaceFill].
	listOrClass == Form
		ifTrue: [editor := UiPushButton new icon: aValue; text: 'Choose...'; hResizing: #spaceFill].
	(listOrClass isKindOf: Collection)
		ifTrue: [editor := UiComboBox new.
			listOrClass do: [:value | editor addText: value asString].
			editor currentIndex: (editor findText: aValue).
			editor hResizing: #spaceFill].
	
	^ editor! !

!UiPropertyTable methodsFor: 'signals' stamp: 'mt 6/30/2010 08:57:11.346'!
focusChanged: aRectangle
	"Tell my scroller what is important for me."
	
	self emit.! !

!UiPropertyTable methodsFor: 'initialization' stamp: 'mt 10/23/2010 08:58:59.26'!
initialize

	super initialize.
	self
		color: (Color gray: 0.95);
		changeTableLayout;
		listDirection: #topToBottom;
		hResizing: #rigid;
		vResizing: #shrinkWrap.
		
	cache := IdentityDictionary new.! !

!UiPropertyTable methodsFor: 'private' stamp: 'mt 6/30/2010 08:32:40.277'!
nextRowColor

	| odd even |
	odd := Color gray: 0.85.
	even := Color gray: 0.95.
	^ nextRowColor
		ifNil: [nextRowColor := odd]
		ifNotNil: [nextRowColor := (nextRowColor = odd
			ifTrue: [even]
			ifFalse: [odd])].! !

!UiPropertyTable methodsFor: 'updating' stamp: 'mt 10/22/2010 11:33:57.869'!
pickIcon: aMorph selector: aSymbol editor: aButton
	
	| codeSnippet |
	codeSnippet := UiIconPickerDialog new
		topLeft: (aButton localPointToGlobal: aButton bottomLeft);
		pickCodeForIcon.

	aMorph perform: aSymbol with: (Compiler evaluate: codeSnippet).
	aMorph setProperty: (aSymbol allButLast, #byDesigner) asSymbol toValue: codeSnippet.
	aButton icon: (Compiler evaluate: codeSnippet).! !

!UiPropertyTable methodsFor: 'updating' stamp: 'mt 7/16/2010 11:43:17.278'!
removeIgnoredProperties: aMorph

	| lastWasSeparator |
	self removeAllMorphsIn: (aMorph class widgetPropertiesToIgnore collect: [:property |
		self submorphs detect: [:row | row submorphs first text = property]]).

	"Remove separators of empty categories."
	lastWasSeparator := false.
	self removeAllMorphsIn: ((self submorphs collect: [:row | |tmp|
		tmp := (row submorphs size = 1 and: [lastWasSeparator]) "Is separator?"
			ifTrue: [row]
			ifFalse: [nil].
		lastWasSeparator := row submorphs size = 1.
		tmp]) reject: [:each | each isNil])! !

!UiPropertyTable methodsFor: 'signals' stamp: 'mt 6/30/2010 08:57:16.963'!
resized
	
	self emit.! !

!UiPropertyTable methodsFor: 'private' stamp: 'mt 7/4/2010 08:46:53.243'!
updateNameColumnWidth
	"To have equal widths."
	
	| widths max |
	widths := Array new: self submorphs size.
	self submorphs withIndexDo: [:row :i |
		row submorphs size > 1 ifTrue: [
			row submorphs first textMorph autoFit: true.
			widths at: i put: row submorphs first textMorph width.
			row submorphs first textMorph autoFit: false]
		ifFalse: [widths at: i put: 0]].

	max := widths max.
	self submorphs do: [:row |
		row submorphs first textMorph width: max.
		"Separators"
		row submorphs size = 1
			ifTrue: [row submorphs first textMorph autoFit: true]].! !

!UiPropertyTable methodsFor: 'updating' stamp: 'mt 10/23/2010 08:58:35.891'!
updateProperties: aMorph

	lastMorph == aMorph ifTrue: [^ self].
	lastMorph := aMorph.
	
	nextRowColor := nil.
	
	self removeAllMorphs.
	
	(cache includesKey: aMorph)
		ifFalse: [self
			addAllProperties: aMorph;
			removeIgnoredProperties: aMorph.
			cache at: aMorph put: self submorphs copy]
		ifTrue: [(cache at: aMorph) do: [:m | self addMorphBack: m]].
	
	self
		updateNameColumnWidth;
		resized.! !

!UiPropertyTable methodsFor: 'scrolling' stamp: 'mt 6/30/2010 09:01:02.836'!
withScrollBar

	^ UiPropertyTableScroller new
		scrollTarget: self! !

Morph subclass: #UiWidgetPageButtonContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Designer-Core'!

!UiWidgetPageButtonContainer commentStamp: 'mt 7/14/2010 11:15:56.239' prior: 0!
I am a container for edit buttons that has a high mouse priority to accept mouse clicks.!

!UiWidgetPageButtonContainer methodsFor: 'events-processing' stamp: 'mt 7/16/2010 12:53:09.338'!
handlerForMouseDown: evt

	^ self submorphs
		detect: [:m | m bounds containsPoint: evt position] ifNone: [self].
		! !

!UiWidgetPageButtonContainer methodsFor: 'events-processing' stamp: 'mt 7/16/2010 12:53:21.162'!
handlerForYellowButtonDown: evt

	^ self submorphs
		detect: [:m | m bounds containsPoint: evt position] ifNone: [self].
		! !

!UiWidgetPageButtonContainer methodsFor: 'initialization' stamp: 'mt 7/16/2010 12:57:41.845'!
initialize

	super initialize.
	self
		color: (Color gray: 0.8);
		changeTableLayout;
		listDirection: #leftToRight;
		borderStyle: (BorderStyle raised width: 1);
		disableTableLayout: true;
		setToAdhereToEdge: #topRight;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.! !

!UiWidgetPageButtonContainer methodsFor: 'events-processing' stamp: 'mt 7/14/2010 11:20:22.466'!
mouseDownPriority

	^ 150! !

Object subclass: #UiCodeGenerator
	instanceVariableNames: 'source category defaultNameCount instVars'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Designer-Core'!

!UiCodeGenerator commentStamp: 'mt 6/27/2010 09:57:56.499' prior: 0!
Generate a subclass for UserInterface based on a given instance of Morph.!

!UiCodeGenerator class methodsFor: 'examples' stamp: 'mt 6/29/2010 19:10:23.021'!
example01

	| ui |
	ui := Morph new
		setProperty: #uiClassName toValue: #Foobar.
	UiCodeGenerator generateUiClassFor: (Morph new
	setProperty: #uiClassName toValue: #Foobar).! !

!UiCodeGenerator class methodsFor: 'instance creation' stamp: 'mt 6/30/2010 11:58:41.695'!
generateUiClassFor: aMorph

	^ self new
		generateUiClassFor: aMorph! !

!UiCodeGenerator class methodsFor: 'instance creation' stamp: 'mt 6/30/2010 11:58:37.693'!
generateUiClassFor: aMorph category: aString

	^ self new
		category: aString;
		generateUiClassFor: aMorph! !

!UiCodeGenerator methodsFor: 'accessing' stamp: 'mt 6/30/2010 11:49:22.277'!
category

	^ category ifNil: ['Widgets-Custom']! !

!UiCodeGenerator methodsFor: 'accessing' stamp: 'mt 6/30/2010 11:49:31.444'!
category: aString

	category := aString.! !

!UiCodeGenerator methodsFor: 'private - inst vars' stamp: 'mt 6/27/2010 12:49:00.982'!
collectInstVars

	instVars := OrderedCollection new.
	self injectInstVarsForMorph: source into: instVars.
	^ instVars! !

!UiCodeGenerator methodsFor: 'code generation' stamp: 'mt 10/22/2010 11:09:30.541'!
createAccessors

	| template |
	template := '{1}
	"{2}"
	
	^ {1}'.
	instVars
		select: [:instVar | instVar at: #hasName]
		thenDo: [:instVar |
		self uiClass compile: (template format: {
			instVar at: #name.
			instVar at: #description}) classified: 'accessing'].! !

!UiCodeGenerator methodsFor: 'code generation' stamp: 'mt 6/30/2010 12:41:44.696'!
createUiClass

	(Smalltalk includesKey: self uiClassName asSymbol)
		ifTrue: [(Smalltalk at: self uiClassName asSymbol)
			removeFromChanges;
			removeFromSystemUnlogged].

	UserInterface subclass: self uiClassName
		instanceVariableNames: self instVarsString
		classVariableNames: ''
		poolDictionaries: ''
		category: self category! !

!UiCodeGenerator methodsFor: 'code generation' stamp: 'mt 7/21/2010 11:27:00.588'!
createUiPanelCode

	| stream uiClassTemplate |
	uiClassTemplate := 'panel setProperty: #uiClassName toValue: ''{1}''.'.
	stream := WriteStream on: ''.
	
	stream nextPutAll: 'initializePanel
	"This code is auto-generated. DO NOT modify it because another generation step will overwrite all changes."
	'.
	
	source setProperty: #objectNameTemp toValue: source objectName.
	self injectUiCodeFor: source into: stream.
	
	"Store UI class name."
	stream crtab; nextPutAll: (uiClassTemplate format: {
		source uiClassName}).
		
	self uiClass
		compile: stream contents
		classified: 'ui - private'.
		! !

!UiCodeGenerator methodsFor: 'code generation' stamp: 'mt 6/30/2010 12:01:06.805'!
generateUiClassFor: aMorph

	source := aMorph.
	
	"This is the root."
	source objectName: #panel.
		
	self createUiClass.
	self createAccessors.
	self createUiPanelCode.! !

!UiCodeGenerator methodsFor: 'initialization' stamp: 'mt 6/27/2010 10:15:31.804'!
initialize

	super initialize.
	defaultNameCount := 0.! !

!UiCodeGenerator methodsFor: 'private - reading' stamp: 'mt 10/22/2010 11:56:27.854'!
injectCustomUiCodeFor: aMorph into: stream

	| propertyTemplate value isCode |
	propertyTemplate := '{1} {2}: ({5}{3}{4}{6}).'.

	aMorph class widgetProperties do: [:category |
		category value do: [:property |
			isCode := false.
			property value == Form
				ifTrue: [value := aMorph valueOfProperty: (property key, #byDesigner) asSymbol ifAbsent: []. isCode := true]
				ifFalse: [value := aMorph perform: property key].

		stream crtab; nextPutAll: (propertyTemplate format: {
			aMorph valueOfProperty: #objectNameTemp.
			property key.
			value isSymbol ifTrue: ['#'] ifFalse: [''].
			value isText ifTrue: [value asString] ifFalse: [value].
			"add string quotation"
			value isSymbol not & isCode not & (value isString | value isText) ifTrue: [''''] ifFalse: [''].
			value isSymbol not & isCode not & (value isString | value isText) ifTrue: [''''] ifFalse: ['']})]].
	
! !

!UiCodeGenerator methodsFor: 'private - inst vars' stamp: 'mt 10/22/2010 11:47:36.431'!
injectInstVarsForMorph: aMorph into: aCollection

	"Root panel already has an instance variable."
	aMorph ~~ source ifTrue: [
		aMorph
			setProperty: #objectNameTemp
			toValue: (aMorph objectName ifEmpty: [self nextDefaultName]).
		
		aMorph
			setProperty: #objectDescroptionTemp
			toValue:  (aMorph objectDescription ifEmpty: ['No description given.']).
		
		aCollection add: (Dictionary newFrom: {
			#name -> (aMorph valueOfProperty: #objectNameTemp).
			#description -> (aMorph valueOfProperty: #objectDescriptionTemp).
			#hasName -> aMorph objectName notEmpty})].
	
	(aMorph isKindOf: UiContainer)
		ifTrue: [
			aMorph submorphs do: [:m | self injectInstVarsForMorph: m into: aCollection]].! !

!UiCodeGenerator methodsFor: 'private - reading' stamp: 'mt 10/19/2010 16:40:32.77'!
injectUiCodeFor: aMorph into: stream

	| creationTemplate submorphTemplate nameTemplate descrTemplate |
	creationTemplate := '{1} := {2} new.'.
	submorphTemplate := '{1} addMorphBack: {2}.'.
	nameTemplate := '{1} setProperty: #objectName toValue: ''{1}''.'.
	descrTemplate := '{1} setProperty: #objectDescription toValue: ''{2}''.'.

	"Create aMorph."
	stream crtab; nextPutAll: (creationTemplate format: {
		aMorph valueOfProperty: #objectNameTemp.
		aMorph class name}).
	
	"Initialize aMorph."
	aMorph hasTableLayout
		ifTrue: [stream crtab; nextPutAll: ('{1} changeTableLayout.' format: {
			aMorph valueOfProperty: #objectNameTemp})].
	
	#(extent position hResizing vResizing cellInset cellPositioning cellSpacing layoutInset listCentering listDirection listSpacing maxCellSize minCellSize wrapCentering wrapDirection) do: [:property |
		self injectUiCodeFor: aMorph property: property into: stream].
	
	self injectCustomUiCodeFor: aMorph into: stream.
	
	aMorph objectName ifNotEmpty: [
		stream crtab; nextPutAll: (nameTemplate format: {
			aMorph objectName.
			aMorph objectName.})].
	aMorph objectDescription ifNotEmpty: [
		stream crtab; nextPutAll: (descrTemplate format: {
			aMorph valueOfProperty: #objectNameTemp.
			aMorph objectDescription.})].
	
	"Let my submorphs initialize."
	(aMorph isKindOf: UiContainer)
		ifTrue: [
			aMorph submorphs do: [:m |
				self injectUiCodeFor: m into: stream.
				stream crtab; nextPutAll: (submorphTemplate format: {
					aMorph valueOfProperty: #objectNameTemp.
					m valueOfProperty: #objectNameTemp.})]].! !

!UiCodeGenerator methodsFor: 'private - reading' stamp: 'mt 10/22/2010 11:57:11.003'!
injectUiCodeFor: aMorph property: aProperty into: stream

	| propertyTemplate value |
	propertyTemplate := '{1} {2}: ({3}{4}).'.

	value := aMorph perform: aProperty.

	stream crtab; nextPutAll: (propertyTemplate format: {
		aMorph valueOfProperty: #objectNameTemp.
		aProperty.
		value isSymbol ifTrue: ['#'] ifFalse: [''].
		value}).! !

!UiCodeGenerator methodsFor: 'private - inst vars' stamp: 'mt 10/22/2010 11:31:31.606'!
instVarsString

	| stream |
	stream := WriteStream on: ''.
	self collectInstVars
		do: [:instVar | stream nextPutAll: (instVar at: #name)]
		separatedBy: [stream nextPut: Character space].
		
	^ stream contents
	! !

!UiCodeGenerator methodsFor: 'accessing' stamp: 'mt 10/19/2010 16:27:50.868'!
nextDefaultName

	defaultNameCount := defaultNameCount + 1.
	^ #unnamedMorph, defaultNameCount asString asSymbol! !

!UiCodeGenerator methodsFor: 'accessing' stamp: 'mt 6/30/2010 12:39:13.791'!
source

	^ source! !

!UiCodeGenerator methodsFor: 'accessing' stamp: 'mt 6/30/2010 12:39:20.016'!
source: aMorph

	source := aMorph! !

!UiCodeGenerator methodsFor: 'accessing' stamp: 'mt 6/27/2010 10:23:59.435'!
uiClass

	^ Smalltalk at: self uiClassName! !

!UiCodeGenerator methodsFor: 'accessing' stamp: 'mt 7/19/2010 08:21:55.907'!
uiClassName

	^ (source uiClassName capitalized, #Ui) asSymbol! !

UiItemModel subclass: #UiWidgetListModel
	instanceVariableNames: 'widgets nameFilter typeFilter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Designer-Core'!

!UiWidgetListModel methodsFor: 'accessing' stamp: 'mt 10/19/2010 14:28:18.977'!
categoryAt: aModelIndex

	aModelIndex parent ifNotNil: [^ self categoryAt: aModelIndex parent].
	
	^ self sortedCategories at: aModelIndex row! !

!UiWidgetListModel methodsFor: 'counts' stamp: 'mt 10/19/2010 13:42:17.033'!
columnCountFor: aParentModelIndex

	^ 1! !

!UiWidgetListModel methodsFor: 'model data' stamp: 'mt 10/19/2010 14:21:38.023'!
iconFor: aModelIndex

	(aModelIndex row = 0 or: [aModelIndex column = 0])
		ifTrue: [^ nil].
	
	self isFiltered ifTrue: [^ (widgets at: aModelIndex row) widgetClassIcon].

	aModelIndex parent ifNotNil: [:parent |
		^ (self widgetAt: aModelIndex) widgetClassIcon].
	
	^ nil! !

!UiWidgetListModel methodsFor: 'index generation' stamp: 'mt 10/19/2010 13:58:35.423'!
indexRow: aRowNumber column: aColumnNumber parent: aParentModelIndex

	^ (UiSimpleModelIndex row: aRowNumber column: aColumnNumber)
		model: self;
		parent: aParentModelIndex! !

!UiWidgetListModel methodsFor: 'initialization' stamp: 'mt 10/19/2010 14:09:13.699'!
initialize

	super initialize.
	self updateWidgets.
	
	SystemChangeNotifier uniqueInstance
		notify: self
		ofSystemChangesOfItem: #class
		using: #processNotification:.! !

!UiWidgetListModel methodsFor: 'testing' stamp: 'mt 10/19/2010 14:18:39.402'!
isFiltered

	^ self nameFilter notNil and: [self nameFilter notEmpty]! !

!UiWidgetListModel methodsFor: 'accessing' stamp: 'mt 7/1/2010 17:38:43.629'!
nameFilter

	^ nameFilter ifNil: [nameFilter := '']! !

!UiWidgetListModel methodsFor: 'accessing' stamp: 'mt 10/19/2010 14:20:28.689'!
nameFilter: aString

	nameFilter := aString.
	self updateWidgets.! !

!UiWidgetListModel methodsFor: 'updating' stamp: 'mt 10/19/2010 14:09:28.021'!
processNotification: event

	self updateWidgets.! !

!UiWidgetListModel methodsFor: 'counts' stamp: 'mt 10/19/2010 14:31:38.223'!
rowCountFor: aParentModelIndex

	self isFiltered ifTrue: [^ aParentModelIndex isNil
		ifTrue: [widgets size]
		ifFalse: [0]].
	
	aParentModelIndex ifNil: [^ widgets keys size].
	
	aParentModelIndex parent ifNotNil: [^ 0].
	
	^ (widgets at: (self sortedCategories at: aParentModelIndex row)) size! !

!UiWidgetListModel methodsFor: 'accessing' stamp: 'mt 10/19/2010 14:30:09.005'!
sortedCategories

	^ (widgets keys copyWithout: Morph widgetCategory) sort, {Morph widgetCategory}! !

!UiWidgetListModel methodsFor: 'model data' stamp: 'mt 10/19/2010 14:44:33.217'!
textFor: aModelIndex

	aModelIndex row = 0 ifTrue: [^ 'Widgets'].
	aModelIndex column = 0 ifTrue: [^ ''].

	self isFiltered ifTrue: [^ (widgets at: aModelIndex row) widgetClassName].
	
	aModelIndex parent ifNotNil: [:parent |
		^ (self widgetAt: aModelIndex) widgetClassName].
	
	^ ('<b>', (self categoryAt: aModelIndex), '</b>') asHtmlText! !

!UiWidgetListModel methodsFor: 'updating' stamp: 'mt 10/19/2010 14:18:58.653'!
updateWidgets

	self isFiltered ifTrue: [^ self updateWidgetsWithFilter].

	widgets := Dictionary new.
	Morph allSubclassesDo: [:c |
		(widgets includesKey: c widgetCategory)
			ifFalse: [(widgets at: c widgetCategory put: SortedCollection new)
				sortBlock: [:i1 :i2 | i1 widgetClassName < i2 widgetClassName]].
		
		(c isWidgetClass or: [c widgetCategory = Morph widgetCategory])
			ifTrue: [(widgets at: c widgetCategory) add: c]].
	
	self modelReset.! !

!UiWidgetListModel methodsFor: 'updating' stamp: 'mt 10/19/2010 14:35:40.366'!
updateWidgetsWithFilter

	widgets := SortedCollection new
		sortBlock: [:i1 :i2 | i1 widgetClassName < i2 widgetClassName].
		
	Morph allSubclassesDo: [:c |
		(c widgetClassName asLowercase includesSubString: self nameFilter asString asLowercase)
			ifTrue: [widgets add: c]].

	self modelReset.! !

!UiWidgetListModel methodsFor: 'accessing' stamp: 'mt 10/19/2010 14:28:30.434'!
widgetAt: aModelIndex

	aModelIndex parent ifNil: [^ nil].
	
	^ (widgets at: (self sortedCategories at: aModelIndex parent row)) at: aModelIndex row! !

SwaGeneralScrollPane subclass: #UiPropertyTableScroller
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Designer-Core'!

!UiPropertyTableScroller methodsFor: 'geometry' stamp: 'mt 6/30/2010 09:06:43.716'!
extent: aPoint

	super extent: aPoint.
	self updateScrollTargetWidth.! !

!UiPropertyTableScroller methodsFor: 'scrolling' stamp: 'mt 6/30/2010 08:55:30.039'!
hScrollbarNeeded

	^ false! !

!UiPropertyTableScroller methodsFor: 'initialize' stamp: 'mt 6/30/2010 09:02:49.773'!
initialize

	super initialize.
	self
		width: UiPropertyTable defaultWidth + self scrollBarThickness; 
		height: 200;
		borderStyle: (BorderStyle inset width: 1).! !

!UiPropertyTableScroller methodsFor: 'scrolling' stamp: 'mt 6/30/2010 09:08:13.4'!
scrollTarget: aMorph

	super scrollTarget: aMorph.
	
	(aMorph isKindOf: UiPropertyTable) ifTrue: [
		self
			connect: aMorph signal: #focusChanged: toSelector: #scrollToShow:;
			connect: aMorph signal: #resized toSelector: #updateScrollbars].! !

!UiPropertyTableScroller methodsFor: 'scrolling' stamp: 'mt 6/30/2010 08:55:30.11'!
scrollToShow: aRectangle
	"Scroll to include as much of aRectangle as possible, where aRectangle is in the scroller's local space."
	
	|offset|
	offset := self scroller offset.
	(aRectangle top - offset y) < 0
		ifTrue: [offset := offset x @ (
					(aRectangle top min: self scrollTarget height - self innerBounds height))].
	(aRectangle bottom - offset y) > self innerBounds height
		ifTrue: [offset := offset x @ (
					(aRectangle top - self innerBounds height + aRectangle height min: self scrollTarget height - self innerBounds height))].
		
	offset = self scroller offset ifFalse: [
		self scroller offset: offset.
		self setScrollDeltas]! !

!UiPropertyTableScroller methodsFor: 'updates' stamp: 'mt 6/30/2010 09:06:31.245'!
updateScrollTargetWidth

	self scrollTarget ifNotNilDo: [:propertyTable |
		propertyTable width: self extent x - 2 -
			(self vScrollbarNeeded
				ifTrue: [self scrollbarThickness]
				ifFalse: [0])].! !

!UiPropertyTableScroller methodsFor: 'updates' stamp: 'mt 7/1/2010 17:21:03.122'!
updateScrollbars

	self updateScrollTargetWidth.
	super updateScrollbars.! !

!UiPropertyTableScroller methodsFor: 'as yet unclassified' stamp: 'mt 7/1/2010 17:20:45.585'!
vScrollbarNeeded

	^ self scrollTarget height > self height! !
